/@
	A simple block device driver

@/
module "Blockdev" : /* this directive sets the global module name */
driver "Blockdev" [ major=113 , blockdevice ] {

	file "globals" /{
		/* Load time variables */
		int %<driver.name>_devs     = 2;     /* 2 devices */
        	int %<driver.name>_rahead   = 2;     /* 2 sector read ahead */
        	int %<driver.name>_size     = 1024;  /* 1 meg each disk */
        	int %<driver.name>_blksize  = 1024;  /* 1kbyte blocks */
        	int %<driver.name>_hardsect = 512;   /* sector size */
	}/

        header "common" /{
        /* this definitions are necessary to switch the blockdevice feature on in linux
           header files :-( */           
   
	extern int %<driver.name>_major;

#       define MAJOR_NR %<driver.name>_major       /* force definitions on in blk.h */
#       define DEVICE_NR(device) MINOR(device)     /* no partition bits */
#       define DEVICE_NAME "%<driver.name>"        /* name for messaging */
/*#     define DEVICE_INTR %<driver.name>_intrptr         *//* pointer to the bottom half */
#       define DEVICE_NO_RANDOM   
#       define DEVICE_OFF(d)

        /* now include blk.h to get the definitions happy */
	#include <linux/blk.h>
	#include <linux/hdreg.h>

	extern	int %<driver.name>_devs ;
       	extern  int %<driver.name>_rahead ;
        extern  int %<driver.name>_size ;
        extern  int %<driver.name>_blksize ;
       	extern  int %<driver.name>_hardsect ; 

	}/


        /*
         * The main module of a driver uses the core method whenever the module
         * is loaded or unloaded to the kernel with insmod
         * the init method will be executed on load and the cleanup method on
         * unload time
         */

	core {  
		init /{ 
			printk("This is a real stupid message\n");
		}/

		cleanup /{
			printk("Have a nice day! \n");
		}/
	}


        /*
         * This method will be called whenever the driver is opened
         */

	open /{
                /* %<driver.name> */
		printk("Stupid module has been opened\n");
	}/

        /*
         * This method will be called whenever the driver is closed
         */
	close /{
		printk("Stupid module has been closed\n");
	}/


        /*
	 * blockdevices have a request function that handles the basic r/w requests to the sectors
         * the CURRENT pointer is set automagically in blk.h
	 */
        request /{
		printk("%<driver.name>: Request called\n");
		while(1) {
			INIT_REQUEST;
                        printk("%<driver.name>_request CURRENT=%p: cmd %i sec %li (nr. %li), next %p\n", CURRENT,
                          CURRENT->cmd,
                          CURRENT->sector,
                          CURRENT->current_nr_sectors,
                          CURRENT->next);

                          switch(CURRENT->cmd) {
                            case READ:
			    printk("%<diver.name>: READ Request \n");
                            /*memcpy(CURRENT->buffer, ptr, size); */
                             break;
                            case WRITE:
			    printk("%<diver.name>: WRITE Request \n");
                            /*memcpy(ptr, CURRENT->buffer, size); */
                            break;
                            default:
                            /* can't happen */
                            end_request(0);
                            continue;
                          }
			end_request(1);
		}
	}/

	ioctl {
             /* from the library this method is simply called with 
              * SetMode( flags );
              */
             user /{

                  case BLKGETSIZE:
		       printk("%<driver.name>: BLKGETSIZE called\n");
                       /* Return the device size, expressed in sectors */
                       if (!arg) return -EINVAL; /* NULL pointer: not valid */
                       /*
                          err=verify_area(VERIFY_WRITE, (long *) arg, sizeof(long));
                          if (err) return err;
                          
                          put_user ( 1024* sbull_sizes[MINOR(inode->i_rdev)]
                          / sbull_hardsects[MINOR(inode->i_rdev)],
                          (long *) arg); 
                       */
                       return 0;

                  case BLKFLSBUF: /* flush */
		       printk("%<driver.name>: BLKFLSBUF called\n");
                       if (!suser()) return -EACCES; /* only root */
                       fsync_dev(inode->i_rdev);
                       invalidate_buffers(inode->i_rdev);
                       return 0;

                  case BLKRAGET: /* return the readahead value */
		       printk("%<driver.name>: BLKRAGET called\n");
                       if (!arg)  return -EINVAL;
                       /*
                       err = verify_area(VERIFY_WRITE, (long *) arg, sizeof(long));
                       if (err) return err;
                       put_user(read_ahead[MAJOR(inode->i_rdev)],(long *) arg);
                       */
                       return 0;

                  case BLKRASET: /* set the readahead value */
		       printk("%<driver.name>: BLKRASET called\n");
                       if (!suser()) return -EACCES;
                       if (arg > 0xff) return -EINVAL; /* limit it */
                       read_ahead[MAJOR(inode->i_rdev)] = arg;
                       return 0;

      		  case BLKRRPART: /* re-read partition table: can't do it */
		       printk("%<driver.name>: BLKRRPART called\n");
        	       return -EINVAL;

                  RO_IOCTLS(inode->i_rdev, arg); /* the default RO operations */

		  
                  case HDIO_GETGEO:
                  /*
                   * get geometry: we have to fake one...  trim the size to a
                   * multiple of 64 (32k): tell we have 16 sectors, 4 heads,
                   * whatever cylinders. Tell also that data starts at sector. 4.
                   */
                  { 
                      struct hd_geometry *geo = (struct hd_geometry *)arg;  
		      int err,size;

                      printk("%<driver.name>: HDIO_GETGEO called\n");
                      if (geo==NULL) return -EINVAL;
                      err = verify_area(VERIFY_WRITE, geo, sizeof(*geo));
                      if (err) return err;
                      size = %<driver.name>_size * 1024 / %<driver.name>_hardsect;
                      size &= ~0x3f; /* multiple of 64 */
                      put_user(size >> 6, &geo->cylinders);
                      put_user(        4, &geo->heads);
                      put_user(       16, &geo->sectors);
                      put_user(        4, &geo->start);
                      return 0;
                  }




	     }/
        }
}


