/@  

   The 'configure' class makes the generation of configuration scripts
   easier. The mechanism is similar to the configure scripts of 
   the linux kernel. Once the scripts are generated you can use 'make config'
   or 'make xconfig' in the new driver source tree. The generated toplevel
   Makefile will use Tcl/Tk (make xconfig) by default unless the --notk option
   is given to ddl2c.
  
   The syntax is:

   configure /{ <code> }/

   while <code> is in the syntax of the linux kernel 'config.in' to define
   interactive option macros.
          
@/


module "Config" :

/*--- This is a demonstration how the configure option works 
 *    the syntax is the same as in the linux kernel configuration
 *    the macros can be used for defining sysctls or other normal
 *    C-macros
 */

configure /{

mainmenu_name "Sysctl Driver Demo Configuration" 
mainmenu_option next_comment  
comment 'Global Driver Options'  
bool 'Build with Testflag' CONFIG_TESTFLAG 
int  'Default Interrupt Level' CONFIG_IRQ 5  
hex  'Default Base address' CONFIG_BASE 230
string 'Default String' CONFIG_STRING "default"
choice 'Processor type' \
        "386            CONFIG_M386     \
         486            CONFIG_M486     \
         Pentium        CONFIG_M586     \
         PPro           CONFIG_M686" Pentium   
endmenu 

mainmenu_option next_comment  
comment 'Misc Options' 
if [ "$CONFIG_TESTFLAG" = "y" ]; then
	int 'Testing Level' CONFIG_LEVEL 3
fi
endmenu

}/
 
driver "Config" [ major=118 ] {



	sysctl {

/*---- global system variables, later accessible through /proc/sys/Sysctl/<name> 
       the default values are stored in normal C defines 
       they are generated through the configure script
*/

		string showit = CONFIG_STRING
		string version = "preliminary test version 0.1"
		int irq   [ access=0444 ] = CONFIG_IRQ
		int[4] base  [ access=0444 ] = CONFIG_BASE

        }

/*----- the driver cores */

	core {  
		init /{			
			printk("init called Default String = '%s'\n",CONFIG_STRING);
		}/

		cleanup /{
			printk("cleanup called\n");
		}/
	}

	open /{
		printk("open called\n");
	}/

	close /{
		printk("close called\n");
	}/

}

